package com.aip.gateway.api.dao.mapper;

import com.aip.gateway.api.entity.*;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

@Mapper
public interface BaseDbMapper {
    List<TbLinkedApiKey> selectLinkedApiKeys();
    List<TbLinkedDecryptKey> selectLinkedDecryptKeys();
    List<TbLinkedSystem> selectLinkedSystems();
    List<TbLinkedServer> selectLinkedServers();
    List<TbAipConfig> selectAipConfig(Map<String, Integer> param);

    int insertAipCallLog(@Param("obj") TbAipCallLog obj);
    int insertAipFileJobLog(@Param("obj") TbAipFileJobLog obj);

}
